@@ -400,6 +400,7 @@ bool Bitmap::TryJPEGCodec(char *buffer, bool &lossyflag) const
       const SequenceOfFragments *sf = PixelData.GetSequenceOfFragments();
       if( !sf ) return false;
       const Fragment &frag = sf->GetFragment(0);
+      if( frag.IsEmpty() ) return false;
       const ByteValue &bv2 = dynamic_cast<const ByteValue&>(frag.GetValue());
       PixelFormat pf = GetPixelFormat(); // PixelFormat::UINT8;
       codec.SetPixelFormat( pf );
@@ -434,13 +435,6 @@ bool Bitmap::TryJPEGCodec(char *buffer, bool &lossyflag) const
               i->GetPixelFormat().SetBitsAllocated( cpf.GetBitsAllocated() );
               i->GetPixelFormat().SetBitsStored( cpf.GetBitsStored() );
               }
-            else if( cpf.GetBitsStored() < pf.GetBitsStored() )
-              {
-              Bitmap *i = const_cast<Bitmap*>(this);
-              gdcmWarningMacro( "Encapsulated stream has more bits actually stored on disk. correcting." );
-              i->GetPixelFormat().SetBitsAllocated( cpf.GetBitsAllocated() );
-              i->GetPixelFormat().SetBitsStored( cpf.GetBitsStored() );
-              }
             }
           }
         }
@@ -684,6 +678,7 @@ bool Bitmap::TryJPEGLSCodec(char *buffer, bool &lossyflag) const
       const SequenceOfFragments *sf = PixelData.GetSequenceOfFragments();
       if( !sf ) return false;
       const Fragment &frag = sf->GetFragment(0);
+      if( frag.IsEmpty() ) return false;
       const ByteValue &bv2 = dynamic_cast<const ByteValue&>(frag.GetValue());
 
       std::stringstream ss;
@@ -731,7 +726,7 @@ bool Bitmap::TryJPEGLSCodec(char *buffer, bool &lossyflag) const
     assert( len <= outbv->GetLength() );
     // DermaColorLossLess.dcm has a len of 63531, but DICOM will give us: 63532 ...
     assert( len <= outbv->GetLength() );
-    if(buffer) memcpy(buffer, outbv->GetPointer(), len /*outbv->GetLength()*/ );  // FIXME
+    memcpy(buffer, outbv->GetPointer(), len /*outbv->GetLength()*/ );  // FIXME
 
     //assert( codec.IsLossy() == ts.IsLossy() );
     lossyflag = codec.IsLossy();
@@ -812,6 +807,7 @@ bool Bitmap::TryJPEG2000Codec(char *buffer, bool &lossyflag) const
       const SequenceOfFragments *sf = PixelData.GetSequenceOfFragments();
       if( !sf ) return false;
       const Fragment &frag = sf->GetFragment(0);
+      if( frag.IsEmpty() ) return false;
       const ByteValue &bv2 = dynamic_cast<const ByteValue&>(frag.GetValue());
 
       bool b = codec.GetHeaderInfo( bv2.GetPointer(), bv2.GetLength() , ts2 );
@@ -846,6 +842,7 @@ bool Bitmap::TryJPEG2000Codec(char *buffer, bool &lossyflag) const
               }
             else if( cpf.GetBitsStored() > pf.GetBitsStored() )
               {
+              // Osirix10vs8BitsStored.dcm
               Bitmap *i = const_cast<Bitmap*>(this);
               gdcmWarningMacro( "Encapsulated stream has more bits actually stored on disk. correcting." );
               i->GetPixelFormat().SetBitsAllocated( cpf.GetBitsAllocated() );
@@ -886,7 +883,7 @@ bool Bitmap::TryJPEG2000Codec(char *buffer, bool &lossyflag) const
     unsigned long check = outbv->GetLength();  // FIXME
     (void)check;
     assert( len <= outbv->GetLength() );
-    if(buffer) memcpy(buffer, outbv->GetPointer(), len /*outbv->GetLength()*/ );  // FIXME
+    memcpy(buffer, outbv->GetPointer(), len /*outbv->GetLength()*/ );  // FIXME
 
     lossyflag = codec.IsLossy();
     if( codec.IsLossy() && !ts.IsLossy() )
